package simuladorvuelos.modelo;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Random;

import colas.modelo.Cola;
/**
 * 
 * Clase que hace la simulacion
 * @author Libardo
 *
 */
public class Simulador {
	public static final int DURACION_DESPEQUE = 5;
	public static final int DURACION_ATERRIZAJE = 10;
	/**
	 * Cola de vuelos
	 */
	private Cola<Vuelo> cola;
	/**
	 * Constructor
	 */
	public Simulador() {
		cola = new Cola<Vuelo>();
		leerVuelosDeArchivo("vuelos.txt");

	}
	/**
	 * Lee la cola de vuelos desde una archivo de texto
	 * @param path
	 */
	private void leerVuelosDeArchivo(String path) {
		File f = new File(path);
		try {
			FileReader fr = new FileReader(f);
			BufferedReader br = new BufferedReader(fr);
			String fila;
			while ((fila = br.readLine()) != null) {
				// String.split(“,”): Parte en campos los datos y los
				// almacena en un array de Strings
				String[] campos = fila.split(",");
				String vuelo = campos[0];
				String aerolinea = campos[1];
				int hora = Integer.parseInt(campos[2]);
				int minutos = Integer.parseInt(campos[3]);
				cola.encolar(new Vuelo(vuelo, aerolinea, hora, minutos));
			}

		} catch (IOException e) {
			System.out.println("Error cargando el archivo: " + e.getMessage());
		} catch (Exception e) {
			System.out.println("Error cargando el archivo: " + e.getMessage());
		}
	}
	/**
	 * Inicia la simulacion
	 */
	public void iniciarSimulacion() {
		if (!cola.esVacia()) {
			Hora hora = cola.frente().getHora();
			hora = new Hora(hora.getHoras(), hora.getMinutos());
			
			while (!cola.esVacia()) {
				if (hayAterrizaje()) {
					System.out.println("Aterrizaje a las " + hora);
					hora.sumar(DURACION_ATERRIZAJE);
				} else {
					Vuelo v = cola.frente();
					cola.desencolar();
					hora = calcularHoraDespegue(hora, v.getHora());
					System.out.println("Despegue, vuelo:" + v.getVuelo()
							+ " Hora programada:" + v.getHora() + " Hora real:"
							+ hora);
					hora.sumar(DURACION_DESPEQUE);
				}
			}
		}
	}
	/**
	 * Dadas la hora actual y la hora programada del vuelo, determina la hora en que debe salir el vuelo
	 * @param horaActual
	 * @param horaProgramada
	 * @return
	 */
	private Hora calcularHoraDespegue(Hora horaActual, Hora horaProgramada) {
		if (horaActual.esMenor(horaProgramada))
			return horaProgramada;
		else
			return horaActual;
	}
	/**
	 * Genera aleatoriamente un aterrizaje
	 * @return true, si hay aterrizaje, false en caso contrario
	 */
	private boolean hayAterrizaje() {
		Random r = new Random();
		return ((r.nextInt() % 2)==1);
	}


}
